/******************************************************************************
* Copyright Kylin Software  Co, Ltd.(c) (2020)
* FileName: user_read.h
* Version:  1.0
* Function Description:
* netflow info test
*
* History:
*   <author>     <time>     <version>    <desc>
*   maguicai  2023/05/19   1.0         Initial build
********************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <arpa/inet.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <linux/udp.h>
#include <stdint.h>
#include <linux/ipv6.h>

#define MAX_PAYLOAD_BYTES 256
#define TASK_COMM_LEN 16
#define MAX_IP_HEADER 40
void print_hex(unsigned char *buf, int len) ;
enum direction { INGRESS, EGRESS };


struct packet_data {
    pid_t pid;
    char comm[TASK_COMM_LEN];
    uint8_t dir; // INGRESS or EGRESS
    uint8_t proto; // IPPROTO_TCP or IPPROTO_UDP
    unsigned char ip_version;   // IPv4 , IPv6 
    int payload_len;
    unsigned char ip_header[MAX_IP_HEADER]; //兼容IPv4和IPv6头,根据前面ip_version字段来区分
    unsigned char tcpudp_header[sizeof(struct tcphdr)]; //兼容两种协议头
    unsigned char payload[MAX_PAYLOAD_BYTES];
};

int main() {
    
     int fd;
    ssize_t ret;
    int buf_len = 200;
    char buf[buf_len * sizeof(struct packet_data)];
    struct packet_data *data;
    char src_ip[INET6_ADDRSTRLEN];
    char dst_ip[INET6_ADDRSTRLEN];
    struct iphdr *iph;
    struct tcphdr *tcph;
    struct udphdr *udph;

    fd = open("/dev/netflow_chardev", O_RDONLY);
    if (fd < 0) {
        perror("open");
        return 1;
    }

    int num = 1000;
    while(1){
        // read返回实际读的数据条目数
        ret = read(fd, &buf, buf_len );
        printf(" read ret = %d  ===== sizeof packet_data %d\n",ret, sizeof(struct packet_data));
            while(ret > 0){
                printf("read buf[%d]\n",ret-1);
                // buf是char数组，取数据时按struct packet_data结构
                data =(struct packet_data *) &buf[(ret-1)*sizeof(struct packet_data)];

                iph = (struct iphdr *)data->ip_header;
                printf("data--->\n");
                if (data->ip_version == 4) {
                    inet_ntop(AF_INET, &((struct iphdr *)iph)->saddr, src_ip, INET_ADDRSTRLEN);
                    inet_ntop(AF_INET, &((struct iphdr *)iph)->daddr, dst_ip, INET_ADDRSTRLEN);
                } else {
                    inet_ntop(AF_INET6, &((struct ipv6hdr *)iph)->saddr, src_ip, INET6_ADDRSTRLEN);
                    inet_ntop(AF_INET6, &((struct ipv6hdr *)iph)->daddr, dst_ip, INET6_ADDRSTRLEN);
                }

                if (data->proto == IPPROTO_TCP) {
                    tcph = (struct tcphdr *)data->tcpudp_header;
                    printf("TCP ");
                } else {
                    udph = (struct udphdr *)data->tcpudp_header;
                    printf("UDP ");
                }

                printf("%s ", data->dir == 1 ? "INGRESS" : "EGRESS");
                printf("PID: %d, Comm: %s, IP_Version:%d Src IP: %s, Dst IP: %s\n",
                    data->pid, data->comm, data->ip_version, src_ip, dst_ip);

                if (data->proto == IPPROTO_TCP) {
                    printf("Src Port: %u, Dst Port: %u\n", ntohs(tcph->source), ntohs(tcph->dest));
                } else {
                    printf("Src Port: %u, Dst Port: %u\n", ntohs(udph->source), ntohs(udph->dest));
                }

                printf("Payload (first %d bytes): ", data->payload_len);
                print_hex(data->payload, data->payload_len);
                printf("\n");
                ret--;
            } 
            // 判断是否读空了数据,读空时停2秒，避免无穷地去请求读
            if(ret < buf_len)
                usleep(10);
            //sleep(2);
       //num--; 
    }

    close(fd);
    return 0;
}
void print_hex(unsigned char *buf, int len) {
    for (int i = 0; i < len; i++) {
        printf("%02x", buf[i]);
    }
}


