/******************************************************************************
* Copyright Kylin Software  Co, Ltd.(c) (2020)
* FileName: user_read_info.h
* Version:  1.0
* Function Description:
* netflow info test
*
* History:
*   <author>     <time>     <version>    <desc>
*   maguicai  2023/05/19   1.0         Initial build
********************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <arpa/inet.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <linux/udp.h>
#include <stdint.h>
#include <linux/ipv6.h>

#define MAX_PAYLOAD_BYTES 256
#define TASK_COMM_LEN 16
#define MAX_IP_HEADER 40
void print_hex(unsigned char *buf, int len) ;
enum direction { INGRESS, EGRESS };

void print_ipv4(struct in_addr ip_addr);


typedef enum {
        IP_VERSION_IPV4 = 0,
        IP_VERSION_IPV6
} ip_version_t;
union ip_addr {
        struct in_addr ipv4;                    // IPv4地址
        struct in6_addr ipv6;                   // IPv6地址
};
struct netflow_proc_tuple{
        union ip_addr src_ip;
        union ip_addr dst_ip;
        uint16_t src_port;
        uint16_t dst_port;
        uint8_t protocol;
        ip_version_t ip_version;
        char process_name[16];
        int  pid;
        unsigned char add;  //增加、删除
        unsigned char dir;  // 1: 出站，0：入站
        char print_ok;
        //struct work_struct work;
};
static void print_tuple_for_test(const struct netflow_proc_tuple *tuple){

    char src_ip[INET6_ADDRSTRLEN] = {0};
    char dst_ip[INET6_ADDRSTRLEN] = {0};

    if (tuple->ip_version == IP_VERSION_IPV4) {
        snprintf(src_ip, sizeof(src_ip), "%pI4", &tuple->src_ip.ipv4);
        snprintf(dst_ip, sizeof(dst_ip), "%pI4", &tuple->dst_ip.ipv4);
        printf(" Src_IP:");
        print_ipv4(tuple->src_ip.ipv4);
        printf(" Dst_IP:");
        print_ipv4(tuple->dst_ip.ipv4);
    } else {
        snprintf(src_ip, sizeof(src_ip), "%pI6", &tuple->src_ip.ipv6);
        snprintf(dst_ip, sizeof(dst_ip), "%pI6", &tuple->dst_ip.ipv6);
    }

    //printf("Src IP: %s, Dst IP: %s\n", src_ip, dst_ip);
    printf("Src port: %u/%u, Dst port: %u/%u\n", ntohs(tuple->src_port),tuple->src_port, ntohs(tuple->dst_port),tuple->dst_port);
    printf("Protocol: %s\n", tuple->protocol == 17 ? "UDP" : "TCP");
    printf("IP version: %s\n", tuple->ip_version == IP_VERSION_IPV4 ? "IPv4" : "IPv6");
    printf("Process name: %s, PID: %d\n", tuple->process_name, tuple->pid);
    printf("Add: %s\n", tuple->add ? "ADD" : "DEL");
    printf("Direction: %s\n", tuple->dir ? "Outbound" : "Inbound");
}
void print_ipv4(struct in_addr ip_addr) {
    char ip_str[INET_ADDRSTRLEN];
    if (inet_ntop(AF_INET, &ip_addr, ip_str, INET_ADDRSTRLEN) == NULL) {
        printf("inet_ntop() error\n");
    } else {
        printf("IP address: %s\n", ip_str);
    }
}

int main() {
    
     int fd;
    ssize_t ret;
    int buf_len = 20;
    char buf[buf_len * sizeof(struct netflow_proc_tuple)];
    struct netflow_proc_tuple *data;
    

    fd = open("/dev/netflow_info", O_RDONLY);
    if (fd < 0) {
        perror("open");
        return 1;
    }

    int num = 1000;
    while(1){
        // read返回实际读的数据条目数
        ret = read(fd, &buf, buf_len );
            while(ret > 0){
        //printf(" read ret = %d  ===== sizeof netflow_proc_tuple %d\n",ret, sizeof(struct netflow_proc_tuple));
         //       printf("read buf[%d]\n",ret-1);
                // buf是char数组，取数据时按struct netflow_proc_tuple
                data =(struct netflow_proc_tuple *) &buf[(ret-1)*sizeof(struct netflow_proc_tuple)];
                print_tuple_for_test(data);
                ret--;
            } 
            // 判断是否读空了数据,读空时停一下，避免无穷地去请求读
            if(ret < buf_len)
                usleep(10);
            //sleep(2);
       //num--; 
    }

    close(fd);
    return 0;
}
void print_hex(unsigned char *buf, int len) {
    for (int i = 0; i < len; i++) {
        printf("%02x", buf[i]);
    }
}


