/******************************************************************************
* Copyright Kylin Software  Co, Ltd.(c) (2020)
* FileName: common.h
* Version:  1.0
* Function Description:
*	A Headers include netflow internal structure
*
* History:
*   <author>     <time>     <version>    <desc>
*   wangshengye  2023/05/19   1.0         Initial build
********************************************************************************/

#ifndef __COMMON_H
#define __COMMON_H

#include <asm/byteorder.h>
#include <linux/types.h>
#include <linux/in.h>
#include <linux/in6.h>

extern int debug;
extern int print_drop_log;

typedef enum {
    NET_FLOW_INBOUND = 0,
    NET_FLOW_OUTBOUND,
    NET_FLOW_ALL
}netflow_dir_t;

typedef enum {
    IP_ADDR_VERSION_IPV4 = 0,
    IP_ADDR_VERSION_IPV6
}ip_version_t;

union ip_addr {
    struct in_addr ipv4;
    struct in6_addr ipv6;
};

struct ip_range {
    union ip_addr start;
    union ip_addr end;
};

struct port_range {
    __be16 start;
    __be16 end;
};

struct net_five_tuple {
    struct ip_range    src_ip_range;      //源IP地址范围，为固定值时，start,end为相同的值
    struct ip_range    dst_ip_range;      //标IP地址范围，为固定值时，start,end为相同的值
    struct port_range  src_port_range;    //源端口范围，为固定值时，start,end为相同的值
    struct port_range  dst_port_range;    //标端口范围，为固定值时，start,end为相同的值
    __u8      protocol;
    ip_version_t  ip_version;
};

struct pkt_addr_info {
    union ip_addr saddr;
    union ip_addr daddr;
    __be16 sport;
    __be16 dport;
    netflow_dir_t dir;
    ip_version_t ip_version;
    __u8 protocol;
};

struct traffic_blocking_log {
    struct pkt_addr_info pkt_info;
    struct net_five_tuple tuple_info;
    volatile unsigned int drop;
    unsigned long long timestamp;
};

//网络流量阻断策略
struct netflow_drop_policy {
    unsigned short int policy_id;       //策略编号
    struct   net_five_tuple tuple;      //网络五元组信息
    netflow_dir_t direction;            //要管控的网络流量方向
};

struct net_flow_policy_info {
    unsigned short int policy_id;       //策略编号 
    struct net_five_tuple tuple_info;   //网络五元组信息
    netflow_dir_t  net_dir;             //要管控的网络流量方向
    struct net_flow_policy_info *next;  //支持链表
};

/*
后面使用该结构体
struct net_flow_policy_info {
    struct netflow_drop_policy policy;
    struct net_flow_policy_info *next;
}

*/

enum POLICY_OPERATION {
    POLICY_ADD,
    POLICY_DEL,
    POLICY_CLEAR,
    POLICY_GETALL,
    POLICY_MAX
};

/*核外与内核通信使用该结构体(添加删除策略等操作)*/
struct policy_recv_msg {
    struct netflow_drop_policy data;
    enum POLICY_OPERATION opt;
};


/*核内接收核外数据之后发送的响应数据*/
struct policy_response_msg {
    int opt_return_code;
    enum POLICY_OPERATION opt;
};


#endif //__COMMON_H
