/******************************************************************************
* Copyright Kylin Software  Co, Ltd.(c) (2020)
* FileName: log_queue.h
* Version:  1.0
* Function Description:
*       Declares the drop log queue operation functions
*
* History:
*    <author>     <time>      <version>    <desc>
*    wangshengye  2023/5/19    1.0         Initial build
********************************************************************************/

#ifndef _LOG_QUEUE_H_
#define _LOG_QUEUE_H_

#include <linux/wait.h>
#include <linux/types.h>
#include "common.h"

/* Define netflow drop log content */
struct netflow_drop_log {
    struct net_five_tuple tuple_info;
    unsigned long timestamp;
    pid_t pid;
    char process_name[16];
    uint16_t policy_id;
    netflow_dir_t direction;
};

/* Define the elements in the queue */
struct log_queue_element {
	struct netflow_drop_log log;
	struct list_head list;
};
/*
 * Define the queue,contains a wait queue header
 */
struct log_queue {
	struct list_head head;
	int len;
	spinlock_t lock;
	wait_queue_head_t wait;
};

/* Declares a queue */
extern struct log_queue log_send_queue;
/* Thread exit flag variable */
extern int kthread_sending_log_exit;
/* Drop log queue initial */
void log_queue_init(void);
/* Put the drop log data in the queue */
int log_enqueue(struct log_queue_element *data);
/* Destroy the queue */
void log_queue_destroy(void);
/* drop log sending function */
int drop_log_sending(void *arg);

#endif //_LOG_QUEUE_H_
