/******************************************************************************
* Copyright Kylin Software  Co, Ltd.(c) (2020)
* FileName: netflow_fs.c
* Version:  1.0
* Function Description:
* netflow fs for test
*
* History:
*   <author>     <time>     <version>    <desc>
*   wangshengye  2023/05/19   1.0         Initial build
********************************************************************************/

#include <linux/module.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/device.h>
#include <linux/kobject.h>
#include "netflow_fs.h"
#include "hashtab.h"
#include "policy_list.h"

static struct kobject *sm_netflow_kobj;

static ssize_t sm_netflow_attr_show(struct kobject *kobj, struct kobj_attribute *attr,
                            char *buf)
{
	u32 len = 0;
	ssize_t total_len = 0;

	struct policy_node *node;
	list_for_each_entry(node, &policy_list_head.head, list) {
		if (node->t.tuple.ip_version == IP_ADDR_VERSION_IPV4) { 
			len = sprintf(buf,"ID:%u, ip_version:%d, protocol:%u, direction:%u, src_ip:%pI4~%pI4(%u~%u), dest_IP:%pI4~%pI4(%u~%u)\n", 
					node->t.policy_id, node->t.tuple.ip_version, node->t.tuple.protocol, node->t.direction,
					&node->t.tuple.src_ip_range.start.ipv4, &node->t.tuple.src_ip_range.end.ipv4,
					ntohs(node->t.tuple.src_port_range.start), ntohs(node->t.tuple.src_port_range.end),
					&node->t.tuple.dst_ip_range.start.ipv4, &node->t.tuple.dst_ip_range.end.ipv4,
					ntohs(node->t.tuple.dst_port_range.start), ntohs(node->t.tuple.dst_port_range.end));
		} else if (node->t.tuple.ip_version == IP_ADDR_VERSION_IPV6) {
			len = sprintf(buf,"ID:%u, ip_version:%d, protocol:%u, direction:%u, src_ip:%pI6~%pI6(%u~%u), dest_IP:%pI6~%pI6(%u~%u)\n", 
					node->t.policy_id, node->t.tuple.ip_version, node->t.tuple.protocol, node->t.direction,
					&node->t.tuple.src_ip_range.start.ipv6, &node->t.tuple.src_ip_range.end.ipv6,
					ntohs(node->t.tuple.src_port_range.start), ntohs(node->t.tuple.src_port_range.end),
					&node->t.tuple.dst_ip_range.start.ipv6, &node->t.tuple.dst_ip_range.end.ipv6,
					ntohs(node->t.tuple.dst_port_range.start), ntohs(node->t.tuple.dst_port_range.end));
		} else {
			len = sprintf(buf,"the ip version is wrong int the policy list!\n");
		}

		buf = buf + len;
		total_len = total_len + len;
		//printk("total_len %lu\n", total_len);
	}

	return total_len;
}

static ssize_t sm_netflow_attr_store(struct kobject *kobj, struct kobj_attribute *attr,
                             const char *buf, size_t count)
{
    return 0;
}

static ssize_t hashtab_info_attr_show(struct kobject *kobj, struct kobj_attribute *attr,
                            char *buf)
{

	struct hashtab_info info;
	memset(&info, 0, sizeof(struct hashtab_info));
	
	hashtab_stat(policy_hashtab, &info);
	
	return sprintf(buf, "slots_used = %u\nmax_chain_len = %u\n", info.slots_used, info.max_chain_len);
}

static ssize_t policy_hit_attr_show(struct kobject *kobj, struct kobj_attribute *attr,
                            char *buf)
{
	u32 i;
	ssize_t len = 0;
	ssize_t total_len = 0;
	struct hashtab_node *cur, *temp;
	struct  pkt_addr_info *ft;

	for (i = 0; i < policy_hashtab->size; i++) {
		cur = policy_hashtab->htable[i];
		while (cur) {
			temp = cur;
			cur = cur->next;
			ft = (struct  pkt_addr_info *)temp->key;
			if (ft->ip_version == IP_ADDR_VERSION_IPV4) {
				len = sprintf(buf, "Policy ID: %u, ip_version: %d, protocol: %d, direction: %d, saddr: %pI4(%u), daddr: %pI4(%u)\n",
								*(unsigned int *)temp->datum, ft->ip_version, ft->protocol, ft->dir,
								&ft->saddr.ipv4, ntohs(ft->sport), &ft->daddr.ipv4, ntohs(ft->dport));
			} else if (ft->ip_version == IP_ADDR_VERSION_IPV6) {
				len = sprintf(buf, "Policy ID: %u, ip_version: %d, protocol: %d, direction: %d, saddr: %pI6(%u), daddr: %pI6(%u)\n",
								*(unsigned int *)temp->datum, ft->ip_version, ft->protocol, ft->dir,
								&ft->saddr.ipv6, ntohs(ft->sport), &ft->daddr.ipv6, ntohs(ft->dport));
			} else {
				len = 0;
			}

			buf = buf + len;
			total_len = total_len + len;
		}
	}
	return total_len;
}

static struct kobj_attribute sm_netflow_attr = __ATTR(policy_list, 0440, sm_netflow_attr_show, sm_netflow_attr_store);
static struct kobj_attribute hashtab_info_attr = __ATTR(hashtab_info, 0440, hashtab_info_attr_show, NULL);
static struct kobj_attribute policy_hit_attr = __ATTR(policy_hit, 0440, policy_hit_attr_show, NULL);

static struct attribute *attrs[] = {
	&sm_netflow_attr.attr,
	&hashtab_info_attr.attr,
	&policy_hit_attr.attr,
	NULL,
};

static struct attribute_group sm_netflow_attr_group = {
    .attrs = attrs,
};

int netflow_fs_init(void)
{
    int ret;

    sm_netflow_kobj = kobject_create_and_add("sm_netflow", kernel_kobj);
    if (!sm_netflow_kobj)
		return -ENOMEM;

    ret = sysfs_create_group(sm_netflow_kobj, &sm_netflow_attr_group);
    if (ret)
        kobject_put(sm_netflow_kobj);

    return ret;
}

void netflow_fs_exit(void)
{
    kobject_put(sm_netflow_kobj);
}

