/******************************************************************************
* Copyright Kylin Software  Co, Ltd.(c) (2020)
* FileName: policy_list.h
* Version:  1.0
* Function Description:
*       Policy list operation function declaration
* History:
*    <author>     <time>      <version>    <desc>
*    wangshengye  2023/5/19    1.0         Initial build
********************************************************************************/
#ifndef _POLICY_LIST_H_
#define _POLICY_LIST_H_

#include <linux/list.h>
#include <linux/spinlock.h>
#include "common.h"

enum RETURN_ERRORS {
	NO_ERROR = 0,
    LIST_EMPTY,
	LIST_FULL,
	NO_FIND_NODE,
	NODE_ALREADY_EXISTS,
	NO_MATCH_POLICY
};

struct policy_node {
    struct list_head list;
    struct netflow_drop_policy t;
};

struct policy_list {
	struct list_head head;
	spinlock_t lock;
	u32 len;
};

/*The drop policy list head*/
extern struct policy_list policy_list_head;

/*Insert a new policy node before the specified head*/
void init_policy_list(struct policy_list *list);

/*Destroy the policy list*/
int destroy_policy_list(struct policy_list *list);

/*Adds a policy to the list*/
int add_drop_policy(struct policy_list *list, struct netflow_drop_policy t);

/*Delete a policy from the list*/
int delete_drop_policy(struct policy_list *policy_list, struct netflow_drop_policy t);

/*Update a policy in the list*/
int update_drop_policy(struct policy_list *list, struct netflow_drop_policy t);

/* Gets all policies in the list */
struct netflow_drop_policy * get_all_policies(struct policy_list *policy_list, unsigned int *size);

/* Clear all policy in the list.*/
int clear_drop_policy(struct policy_list *list);

/*Matches the policy by traversing the list*/
unsigned int match_drop_polciy(struct policy_list *policy_list, struct pkt_addr_info pkt);

#endif //_POLICY_LIST_H_
